import sys
from argparse import ArgumentParser, Namespace


def add_kwargs(argument_parser: ArgumentParser):
    args = [arg for arg in sys.argv[3:] if not arg.startswith("--env")]

    for arg in args:
        arg_parsed = arg.split("=")

        argument_parser.add_argument(arg_parsed[0])


def extract_kwargs(input_args: Namespace):
    values = input_args.__dict__

    values.pop("help_selected", None)
    values.pop("command_name", None)

    return values



import sys
from argparse import ArgumentParser, Namespace
from logging import Logger
from box import Box
from consolebundle.ConsoleCommand import ConsoleCommand
from jobsbundle.job import argparser_configurator
from databricks_cli.jobs.api import JobsApi
from jobsbundle.job.JobIdFinder import JobIdFinder
from jobsbundle.job.Router import Router
from jobsbundle.job.ValuesFiller import ValuesFiller


class JobCreatorCommand(ConsoleCommand):
    def __init__(
        self,
        jobs_raw_config: Box,
        logger: Logger,
        jobs_api: JobsApi,
        job_id_finder: JobIdFinder,
        values_filler: ValuesFiller,
        router: Router,
    ):
        self.__jobs_raw_config = jobs_raw_config
        self.__logger = logger
        self.__jobs_api = jobs_api
        self.__job_id_finder = job_id_finder
        self.__values_filler = values_filler
        self.__router = router

    def get_command(self) -> str:
        return "dbx:job:create"

    def get_description(self):
        return "Create a new Databricks job based on given job identifier"

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument(dest="identifier", help="Job identifier")
        argparser_configurator.add_kwargs(argument_parser)

    def run(self, input_args: Namespace):
        if input_args.identifier not in self.__jobs_raw_config:
            self.__logger.error(
                f"No job found for {input_args.identifier}. Maybe you forgot to add the configuration under jobsbundle.jobs?"
            )
            sys.exit(1)

        job_raw_config = self.__jobs_raw_config[input_args.identifier].to_dict()
        values = argparser_configurator.extract_kwargs(input_args)
        job_config = self.__values_filler.fill(job_raw_config, values, input_args.identifier)

        job_id = self.__job_id_finder.find(job_config.name)

        if job_id:
            job_url = self.__router.generate_url("job.detail", job_id=job_id)
            self.__logger.error(f'Job with name "{job_config.name}" already exist, exiting {job_url}')
            sys.exit(1)

        self.__logger.info(f'Creating job {input_args.identifier} with name "{job_config.name}"')

        self.__jobs_api.create_job(job_config)

        job_url = self.__router.generate_url("job.detail", job_id=job_id)
        self.__logger.info(f"Job successfully created {job_url}")



from argparse import ArgumentParser, Namespace
from logging import Logger
from box import Box
import time
from consolebundle.ConsoleCommand import ConsoleCommand
from jobsbundle.job import argparser_configurator
from jobsbundle.job.Router import Router
from jobsbundle.job.ValuesFiller import ValuesFiller
from databricks_cli.jobs.api import JobsApi
from databricks_cli.runs.api import RunsApi
from jobsbundle.job.JobIdFinder import JobIdFinder


class StreamingJobCreateOrUpdateCommand(ConsoleCommand):
    def __init__(
        self,
        jobs_raw_config: Box,
        logger: Logger,
        jobs_api: JobsApi,
        runs_api: RunsApi,
        values_filler: ValuesFiller,
        job_id_finder: JobIdFinder,
        router: Router,
    ):
        self.__jobs_raw_config = jobs_raw_config
        self.__logger = logger
        self.__jobs_api = jobs_api
        self.__runs_api = runs_api
        self.__values_filler = values_filler
        self.__job_id_finder = job_id_finder
        self.__router = router

    def get_command(self) -> str:
        return "dbx:job:streaming-create-or-update"

    def get_description(self):
        return "Creates new or updates existing Databricks streaming job based on given job identifier and cancels the active run if exists"

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument(dest="identifier", help="Job identifier")
        argparser_configurator.add_kwargs(argument_parser)

    def run(self, input_args: Namespace):
        if input_args.identifier not in self.__jobs_raw_config:
            self.__logger.error(
                f"No job found for {input_args.identifier}. Maybe you forgot to add the configuration under jobsbundle.jobs?"
            )
            return

        job_raw_config = self.__jobs_raw_config[input_args.identifier].to_dict()
        values = argparser_configurator.extract_kwargs(input_args)
        job_config = self.__values_filler.fill(job_raw_config, values, input_args.identifier)

        job_id = self.__job_id_finder.find(job_config.name)

        if job_id:
            self.__logger.info(f"Existing job found with ID: {job_id}, updating")
            self.__jobs_api.reset_job({"job_id": job_id, "new_settings": job_config.to_dict()})
            job_url = self.__router.generate_url("job.detail", job_id=job_id)
            self.__logger.info(f"Job successfully updated {job_url}")
            self.__cancel_active_run(job_id)
        else:
            self.__logger.info(f'No existing job with name "{job_config.name}" found, creating new one')
            job_id = self.__jobs_api.create_job(job_config.to_dict())["job_id"]
            job_url = self.__router.generate_url("job.detail", job_id=job_id)
            self.__logger.info(f"Job successfully created {job_url}")

        self.__jobs_api.run_now(job_id)
        self.__logger.info("Job successfully started")

    def __cancel_active_run(self, job_id: str):
        self.__logger.info("Looking for active runs...")
        runs_active = self.__runs_api.list_runs(job_id=job_id, active_only=True)
        while "runs" in runs_active:
            for run in runs_active["runs"]:
                run_id = run["run_id"]
                self.__runs_api.cancel_run(run_id=run_id)
                self.__logger.info(f"Run {run_id} canceled")

                time.sleep(5)
                runs_active = self.__runs_api.list_runs(job_id=job_id, active_only=True)
        self.__logger.info("No active run exists")



from box import Box
from jobsbundle.job.template_filler import fill_template


class ValuesFiller:
    def fill(self, template: dict, values: dict, identifier: str) -> Box:
        values["identifier"] = identifier

        def fill_dict_template(value):
            if isinstance(value, dict):
                return {k: fill_dict_template(v) for k, v in value.items()}

            if isinstance(value, list):
                return list(map(fill_dict_template, value))

            if isinstance(value, str):
                return fill_template(value, values)

            return value

        return Box({k: fill_dict_template(v) for k, v in template.items()})



from argparse import ArgumentParser, Namespace
from logging import Logger
from box import Box
from consolebundle.ConsoleCommand import ConsoleCommand
from jobsbundle.job import argparser_configurator
from jobsbundle.job.Router import Router
from jobsbundle.job.ValuesFiller import ValuesFiller
from databricks_cli.jobs.api import JobsApi


class StreamingJobCreateCommand(ConsoleCommand):
    def __init__(self, jobs_raw_config: Box, logger: Logger, jobs_api: JobsApi, values_filler: ValuesFiller, router: Router):
        self.__jobs_raw_config = jobs_raw_config
        self.__logger = logger
        self.__jobs_api = jobs_api
        self.__values_filler = values_filler
        self.__router = router

    def get_command(self) -> str:
        return "dbx:job:streaming-create"

    def get_description(self):
        return "Creates new Databricks streaming job based on given job identifier"

    def configure(self, argument_parser: ArgumentParser):
        argument_parser.add_argument(dest="identifier", help="Job identifier")
        argparser_configurator.add_kwargs(argument_parser)

    def run(self, input_args: Namespace):
        if input_args.identifier not in self.__jobs_raw_config:
            self.__logger.error(
                f"No job found for {input_args.identifier}. Maybe you forgot to add the configuration under jobsbundle.jobs?"
            )
            return

        job_raw_config = self.__jobs_raw_config[input_args.identifier].to_dict()
        values = argparser_configurator.extract_kwargs(input_args)
        job_config = self.__values_filler.fill(job_raw_config, values, input_args.identifier)

        job_id = self.__jobs_api.create_job(job_config.to_dict())["job_id"]
        job_url = self.__router.generate_url("job.detail", job_id=job_id)
        self.__logger.info(f"Job successfully created {job_url}")

        self.__jobs_api.run_now(job_id)
        self.__logger.info("Job successfully started")



from logging import Logger
from databricks_cli.jobs.api import JobsApi


class JobIdFinder:
    def __init__(
        self,
        logger: Logger,
        jobs_api: JobsApi,
    ):
        self.__logger = logger
        self.__jobs_api = jobs_api

    def find(self, job_name: str):
        jobs_response = self.__jobs_api.list_jobs()

        if "jobs" not in jobs_response:
            self.__logger.info("No jobs exist")
            return None

        jobs = jobs_response["jobs"]
        for job in jobs:
            if job["settings"]["name"] == job_name:
                return job["job_id"]
        return None



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['jobsbundle', 'jobsbundle.job']

package_data = \
{'': ['*'], 'jobsbundle': ['_config/*']}

install_requires = \
['console-bundle>=0.5,<0.6',
 'databricks-cli>=0.16.0',
 'pyfony-bundles>=0.4.0,<0.5.0',
 'python-box>=3.4,<4.0']

entry_points = \
{'pyfony.bundle': ['create = jobsbundle.JobsBundle:JobsBundle']}

setup_kwargs = {
    'name': 'jobs-bundle',
    'version': '0.4.0.dev2',
    'description': 'Databricks jobs management bundle for the Daipe Framework',
    'long_description': '# Databricks jobs bundle\n\nCreate Databricks jobs from YAML templates.\n\nProvides console commands to automate jobs creation, deletion, ...\n\n## Installation\n\nInstall the bundle via Poetry:\n\n```\n$ poetry add jobs-bundle\n```\n',
    'author': 'Jiri Koutny',
    'author_email': 'jiri.koutny@datasentics.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/daipe-ai/jobs-bundle',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



class Router:
    def __init__(
        self,
        databricks_url: str,
        routes: dict,
    ):
        self.__databricks_url = databricks_url
        self.__routes = routes

    def generate_url(self, route_name: str, **kwargs):
        if route_name not in self.__routes:
            raise Exception(f"Route not defined: {route_name}")

        databricks_url = self.__databricks_url if self.__databricks_url[-1] != "/" else self.__databricks_url[:-1]

        return databricks_url + self.__routes[route_name].format(**kwargs)
